global gSoundChannels, gSoundPath, gsoundLevel, gKeyFrameSound, gKeyFrameSoundMode, gEnvironmentSound, gLastSoundChannel, gCurKeyList, sQTCharSprite, sQTSPRITE

on setAmbientSound
  set snd to getaProp(gCurKeyList, #Audio)
  set gKeyFrameSound to getaProp(snd, #enter)
  set gKeyFrameSoundMode to getaProp(snd, #exit)
  set theEnvironmentSound to getaProp(snd, #loop)
  if theEnvironmentSound = #NULL then
    stopSound(2)
  else
    if theEnvironmentSound <> #noLoop then
      playSound(string(theEnvironmentSound), #ambientLoop)
    else
      nothing()
    end if
  end if
  if gKeyFrameSound <> #noEnter then
    playSound(string(gKeyFrameSound), #keyframeFile)
    if gKeyFrameSoundMode = #KFSOUND then
      alert("bad sound definition. gKeyFrameSoundMode = #KFSOUND")
      playSound(string(gKeyFrameSound), #keyframeFile)
    else
      if gKeyFrameSoundMode = #LOOPEDKFSOUND then
        alert("bad sound definition. gKeyFrameSoundMode = #LOOPEDKFSOUND")
        playSound(string(gKeyFrameSound), #keyframeFileLooped)
      end if
    end if
  end if
end

on serviceSound
  repeat with Channel = 1 to 8
    set theSoundData to getaProp(gSoundChannels, Channel)
    if not voidp(theSoundData) then
      if the status of theSoundData = #paused then
        next repeat
      end if
      if getOne([#ambientLoop, #keyframeFileLooped, #FileLooped, #animatorFile], the type of theSoundData) > 0 then
        if not soundBusy(Channel) then
          sound playFile Channel, gSoundPath & the name of theSoundData
        end if
        next repeat
      end if
      if (the status of theSoundData = #playing) and not soundBusy(Channel) then
        deleteProp(gSoundChannels, Channel)
      end if
    end if
  end repeat
end

on setSoundVolumes
  set the soundLevel to gsoundLevel
  set relativeVolume to gsoundLevel * 36
  set the volume of sprite sQTCharSprite to relativeVolume
  set the volume of sprite sQTSPRITE to relativeVolume
  repeat with Channel in [2, 3, 4, 5, 6, 7, 8]
    set the volume of sound Channel to relativeVolume
  end repeat
  set theSoundData to getaProp(gSoundChannels, 1)
  if not voidp(theSoundData) then
    set the volume of sound 1 to relativeVolume
  end if
end

on stopSound nameOrChannelToStop
  if nameOrChannelToStop = #ALL then
    puppetSound(0)
    repeat with Channel = 1 to 8
      sound stop Channel
    end repeat
    exit
  else
    if integerp(nameOrChannelToStop) then
      if nameOrChannelToStop = 1 then
        puppetSound(0)
      end if
      sound stop nameOrChannelToStop
      deleteProp(gSoundChannels, nameOrChannelToStop)
      exit
    else
      repeat with Channel = 1 to 8
        set theSoundData to getaProp(gSoundChannels, Channel)
        if not voidp(theSoundData) then
          set name to the name of theSoundData
          if name = nameOrChannelToStop then
            if Channel = 1 then
              puppetSound(0)
            end if
            sound stop Channel
            deleteProp(gSoundChannels, Channel)
            exit
          end if
        end if
      end repeat
    end if
  end if
end

on stopNonAmbientSounds
  repeat with Channel in [1, 3, 4, 5, 6, 7, 8]
    set theSoundData to getaProp(gSoundChannels, Channel)
    if not voidp(theSoundData) then
      if getOne([#keyframeCast, #keyframeFile, #keyframeFileLooped, #animatorCast, #animatorFile], the type of theSoundData) > 0 then
        deleteProp(gSoundChannels, Channel)
        sound stop Channel
      end if
    end if
  end repeat
end

on SelectPlayChannel type, name
  if type = #ambientLoop then
    set Channel to 2
    return Channel
  else
    set freeChannel to 0
    repeat with Channel = 8 down to 4
      set theSoundData to getaProp(gSoundChannels, Channel)
      if voidp(theSoundData) then
        set freeChannel to Channel
        next repeat
        next repeat
      end if
      if getaProp(theSoundData, #name) = name then
        set freeChannel to Channel
        exit repeat
        next repeat
      end if
      if not soundBusy(Channel) and (not getOne([#ambientLoop, #FileLooped, #animatorCast, #animatorFile, #animatorFileLooped], type) > 0) then
        set freeChannel to Channel
        next repeat
      end if
    end repeat
    return freeChannel
  end if
end

on CheckChannelStatus Channel, name
  set theSoundData to getaProp(gSoundChannels, Channel)
  if not voidp(theSoundData) then
    if (getaProp(theSoundData, #name) = name) and (getaProp(theSoundData, #status) = #playing) then
      return #playing
    end if
  else
    return #true
  end if
end

on playSound name, type, bWaitFor
  set Start to the timer
  set Channel to SelectPlayChannel(type, name)
  if Channel <> 0 then
    set status to CheckChannelStatus(Channel, name)
    if status = #playing then
      set gLastSoundChannel to Channel
      return Channel
    end if
    setaProp(gSoundChannels, Channel, [#name: name, #type: type, #status: #playing])
    if getOne([#cast, #animatorCast, #keyframeCast], type) > 0 then
      puppetSound(Channel, cast the number of cast name)
    else
      if getOne([#ambientLoop, #File, #FileLooped, #animatorFile, #keyframeFile, #keyframeFileLooped], type) > 0 then
        sound playFile Channel, gSoundPath & name
      else
        displayChaosError(#info, "Sound type undefined:" && string(type))
        exit
      end if
    end if
  else
    displayChaosError(#warning, "Ran out of sound channels attempting to play:" && name)
    exit
  end if
  if (bWaitFor = #NOW) and (Channel <> 0) then
    updateStage()
  else
    if bWaitFor and (Channel <> 0) then
      cursor([the number of cast "NullCursor", the number of cast "NullCursor.mask"])
      updateStage()
      repeat while soundBusy(Channel)
        cursorandupdate()
      end repeat
    end if
  end if
  set gLastSoundChannel to Channel
  return Channel
end

on waitForSoundDone Channel
  repeat while soundBusy(Channel)
    cursorandupdate()
  end repeat
end

on setSoundState action
  if action = #pause then
    puppetSound(0)
    repeat with Channel = 1 to 8
      set theSoundData to getaProp(gSoundChannels, Channel)
      if not voidp(theSoundData) then
        set the status of getaProp(gSoundChannels, Channel) to #paused
        sound stop Channel
      end if
    end repeat
    exit
  end if
  if action = #play then
    repeat with Channel = 1 to 8
      set theSoundData to getaProp(gSoundChannels, Channel)
      if not voidp(theSoundData) then
        if the type of theSoundData = #keyframeCast then
          deleteProp(gSoundChannels, Channel)
          next repeat
        end if
        playSound(the name of theSoundData, the type of theSoundData)
      end if
    end repeat
    exit
  end if
  displayChaosError(#info, "Sound state undefined:" && string(action))
end

on displayChaosError type, message
  if type = #info then
    set errorMessage to "Chaos Error - Information" & RETURN & RETURN & message
  else
    if type = #warning then
      set errorMessage to "Chaos Error - Warning" & RETURN & RETURN & message
    else
      if type = #severe then
        set errorMessage to "Chaos Error - Severe" & RETURN & RETURN & message & RETURN & RETURN & "You should probably quit now."
      end if
    end if
  end if
  alert(errorMessage)
end

on playSoundTerry name, type, bWaitFor
  if type = #ambientCast then
    set Channel to 1
    puppetSound(name)
    set theSoundData to [:]
    addProp(theSoundData, #name, name)
    addProp(theSoundData, #type, type)
    addProp(theSoundData, #status, #playing)
    setaProp(gSoundChannels, Channel, theSoundData)
    if bWaitFor then
      repeat while soundBusy(Channel)
        cursorandupdate()
      end repeat
    end if
    set gLastSoundChannel to Channel
    return Channel
  else
    if type = #ambientLoop then
      set Channel to 2
      set theSoundData to getaProp(gSoundChannels, Channel)
      if voidp(theSoundData) then
        set theSoundData to [:]
        addProp(theSoundData, #name, name)
        addProp(theSoundData, #type, type)
        addProp(theSoundData, #status, #playing)
        setaProp(gSoundChannels, Channel, theSoundData)
      else
        if the name of theSoundData = name then
          if the status of theSoundData = #playing then
            if bWaitFor then
              waitForSoundDone(Channel)
            end if
            set gLastSoundChannel to Channel
            return Channel
          else
            set the status of getaProp(gSoundChannels, Channel) to #playing
          end if
        else
          set the name of getaProp(gSoundChannels, Channel) to name
          set the type of getaProp(gSoundChannels, Channel) to type
          set the status of getaProp(gSoundChannels, Channel) to #playing
        end if
      end if
      sound playFile Channel, gSoundPath & name
      if bWaitFor then
        waitForSoundDone(Channel)
      end if
      set gLastSoundChannel to Channel
      return Channel
    else
      if (type = #keyframeFile) or (type = #keyframeFileLooped) or (type = #keyframeCast) then
        set Channel to 3
        set theSoundData to getaProp(gSoundChannels, Channel)
        if voidp(theSoundData) then
          set theSoundData to [:]
          addProp(theSoundData, #name, name)
          addProp(theSoundData, #type, type)
          addProp(theSoundData, #status, #playing)
          addProp(gSoundChannels, Channel, theSoundData)
        else
          if the name of theSoundData = name then
            if the status of theSoundData = #playing then
              if bWaitFor then
                waitForSoundDone(Channel)
              end if
              set gLastSoundChannel to Channel
              return Channel
            else
              set the status of getaProp(gSoundChannels, Channel) to #playing
            end if
          else
            set the name of getaProp(gSoundChannels, Channel) to name
            set the type of getaProp(gSoundChannels, Channel) to type
            set the status of getaProp(gSoundChannels, Channel) to #playing
          end if
        end if
        if type = #keyframeCast then
          puppetSound(Channel, cast the number of cast name)
        else
          sound playFile Channel, gSoundPath & name
        end if
        if bWaitFor then
          waitForSoundDone(Channel)
        end if
        set gLastSoundChannel to Channel
        return Channel
      else
        if getOne([#File, #FileLooped, #cast, #animatorCast, #ambientLoop, #animatorFileLooped], type) > 0 then
          set freeChannel to 0
          repeat with Channel = 4 to 8
            set theSoundData to getaProp(gSoundChannels, Channel)
            if not voidp(theSoundData) then
              if (the name of theSoundData = name) and (the type of theSoundData = type) then
                if the status of theSoundData = #playing then
                  if bWaitFor then
                    waitForSoundDone(Channel)
                  end if
                  set gLastSoundChannel to Channel
                  return Channel
                else
                  set the status of getaProp(gSoundChannels, Channel) to #playing
                  if bWaitFor then
                    waitForSoundDone(Channel)
                  end if
                  set gLastSoundChannel to Channel
                  return Channel
                end if
              else
                if (the status of theSoundData = #playing) and not soundBusy(Channel) and (the type of theSoundData <> #FileLooped) and (the type of theSoundData <> #animatorFileLooped) then
                  set freeChannel to Channel
                  exit repeat
                end if
              end if
              next repeat
            end if
            set freeChannel to Channel
            exit repeat
          end repeat
          if freeChannel > 0 then
            set theSoundData to getaProp(gSoundChannels, freeChannel)
            if voidp(theSoundData) then
              set theSoundData to [:]
              addProp(theSoundData, #name, name)
              addProp(theSoundData, #type, type)
              addProp(theSoundData, #status, #playing)
              addProp(gSoundChannels, freeChannel, theSoundData)
            else
              set the name of getaProp(gSoundChannels, freeChannel) to name
              set the type of getaProp(gSoundChannels, freeChannel) to type
              set the status of getaProp(gSoundChannels, freeChannel) to #playing
            end if
          else
            displayChaosError(#warning, "Ran out of sound channels attempting to play:" && name)
            set gLastSoundChannel to Channel
            return 0
          end if
          if getOne([#cast, #animatorCast, #keyframeCast], type) > 0 then
            puppetSound(freeChannel, cast the number of cast name)
          else
            sound playFile freeChannel, gSoundPath & name
          end if
          if bWaitFor then
            waitForSoundDone(Channel)
          end if
          set gLastSoundChannel to Channel
          return freeChannel
        else
          displayChaosError(#info, "Sound type undefined:" && string(type))
          set gLastSoundChannel to Channel
          return 0
        end if
      end if
    end if
  end if
end

on checkAmbientsound
end
